# FT991用 バンド/モード/受信帯域/CW速度/スケルチのコントローラー
# version 016
# 2024/07/01
# programmed by Toshi JA0RUG

import tkinter as tk
import tkinter.messagebox as txtmsg
import threading
import serial
import time

COM  = 'com8'  # COMポート番号
BAUD = 38400   # ボーレート

FONT_SIZE = 20 # 表示フォントサイズ

WPM = 25       # CW速度初期値
polling = 1    # 問い合わせ周期
squel = 5      # スケルチ動作レベル

mode  = ''   # MODEの記憶用変数
wpm = WPM    

exit_flag = False
rit_flag = False # 初期値 RITはOFF
xit_flag = False # 初期値 XITはOFF

MODE = ['LSB', 'USB', 'AM', 'CW', 'FM','RTTY', 'C4FM', 'DATA-USB', '']
BAND = ['1.8', '3.5', '7.0', '10', '14', '18', '21', '24', '28', '50', '144', '430']

try:
    ser = serial.Serial(COM, BAUD)
except:
    txtmsg.showerror('COMポートオープンエラー', COM +'がありません.')
    exit()

root = tk.Tk()
root.title('FT-991 Controller')
root.option_add('*Button.font', ('', FONT_SIZE, 'bold')) # ボタンフォントサイズ
root.resizable(False, False)
root.attributes("-toolwindow", True)

var1 = tk.StringVar() # WPM
var2 = tk.StringVar() # FREQ
var3 = tk.StringVar() # MODE
var4 = tk.StringVar() # SQUELCH

# CW SPEED初期値設定
def init():
    var1.set(f'{WPM} WPM')
    send_cmd(f'KS{WPM:03};'.encode())

# CW SPEED初期値再設定
def reset():
    global wpm

    wpm = WPM
    init()

def read_cmd():
    global mode, squel

    while True:

        try:
            cat_cmd = ser.read_until(b';').decode() 

            if cat_cmd[0:2] == 'IF' and cat_cmd[-1] == ';':

                #FT991/FT991A
                mode = cat_cmd[21]
                freq = (f'{cat_cmd[5:8]}.{cat_cmd[8:11]}.{cat_cmd[11:13]}').lstrip('0')

                var2.set(freq)
                var3.set(mode_num_2_str(mode))

            elif cat_cmd[0:2] == 'SQ' and cat_cmd[-1] == ';':
                squel = int(cat_cmd[3:6])
                var4.set(f'SQL{squel: 3}')

        except:
            break

def send_cmd(cmd):

    try:
        ser.write(cmd)

    except:

        if exit_flag == True:
            return

        txtmsg.showerror('COMポートエラー', COM +'が切断しました.')
        root.destroy()

def send_if_cmd():

    while True:

        if exit_flag == True:
            return

        try:
            send_cmd(b'IF;') 
            time.sleep(polling)

        except:
            return

def mode_num_2_str(mode_nr):

    match mode_nr:

        case '1':
            return 'LSB'

        case '2':
            return 'USB'

        case '3':
            return 'CW'

        case '4':
            return 'FM'

        case '5':
            return 'AM'

        case '6':
            return 'RTTY'

        case 'C':
            return 'DATA-USB'

        case 'E':
            return 'C4FM'

        case '7':
            return 'CW-R'

        case '8':
            return 'DATA-LSB'

        case '9':
            return 'RTTY-USB'

        case 'A':
            return 'DATA-FM'

        case 'B':
            return 'FM-N'

        case 'D':
            return 'AM-N'

def set_mode(mode):
    send_cmd(mode)

def set_band(band_cmd):
    send_cmd(band_cmd)

def ez_split_on(e):

    if xit_flag != True: # XIT ON
        set_xit()

    send_cmd(b'RT1;') # Set RIT ON
    rit_button['bg'] = 'green2'

def ez_split_off(e):
    global rit_flag

    send_cmd(b'RT0;') # Set RIT OFF
    rit_button['bg'] = 'SystemButtonFace'
    rit_flag = False

def set_rit():
    global rit_flag

    if rit_flag == False:
        send_cmd(b'RT1;') # Set RIT ON
        rit_button['bg'] = 'green2'
        rit_flag = True

    else:
        send_cmd(b'RT0;') # Set RIT OFF
        rit_button['bg'] = 'SystemButtonFace'
        rit_flag = False

def set_xit():
    global xit_flag

    if xit_flag == False:
        send_cmd(b'XT1;') # Set RIT ON
        xit_button['bg'] = 'maroon1'
        xit_flag = True

    else:
        send_cmd(b'XT0;') # Set RIT OFF
        xit_button['bg'] = 'SystemButtonFace'
        xit_flag = False

def clear():

    send_cmd(b'RC;') # RIT/XIT Freq Clear

def set_narrow(width):

# Narrowを有効にするモード
# 1:LSB 2:USB 3:CW　6:RTTY-L

    if mode == '1' or mode == '2' or mode == '3' or mode == '6':

        if width == 'n':
            send_cmd(b'NA01;') # Narrow ON
 
        else:
            send_cmd(b'NA00;') # Narrow OFF

def set_sq():
    if mode == '4':
        send_cmd(b'SQ0;')
        var4.set(f'SQL{squel: 3}')

def wheel1(event):

    global wpm

    if event.delta > 0:
        wpm += 1

        if wpm > 60:
            wpm = 60

    else:
        wpm -= 1

        if wpm < 4:
            wpm = 4

    var1.set(f'{wpm: 2} WPM')
    send_cmd(f'KS{wpm:03};'.encode())

def wheel2(event):

    global squel

    if event.delta > 0:
        squel += 1

        if squel > 100:
            squel = 100

    else:
        squel -= 1

        if squel < 1:
            squel = 1

    if mode == '4':
        var4.set(f'SQL{squel: 3}')
        send_cmd(f'SQ{squel:04};'.encode())

fa = tk.Frame(root, bg = 'black')
fa.pack(fill=tk.X)

label_1 = tk.Label(fa, textvariable=var2, width=16, fg='white', bg = 'black', font=('', FONT_SIZE+8, 'bold')).pack(side=tk.LEFT) # FREQ
label_2 = tk.Label(fa, textvariable=var3, width=8,  fg='white', bg = 'black', font=('', FONT_SIZE+6, 'bold')).pack(side=tk.LEFT) # MODE

fb = tk.Frame(root)
fb.pack()

f1  = tk.Frame(fb)
f1.grid(column=1, row=0)

f2  = tk.Frame(fb)
f2.grid(column=2, row=0)

f3  = tk.Frame(fb)
f3.grid(column=3, row=0)

f4  = tk.Frame(fb)
f4.grid(column=4, row=0)

# modeボタン設定
tk.Button(f1, text = MODE[0], command = lambda: set_mode(b'MD01;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # LSB
tk.Button(f2, text = MODE[1], command = lambda: set_mode(b'MD02;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # USB
tk.Button(f3, text = MODE[2], command = lambda: set_mode(b'MD05;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # AM

tk.Button(f1, text = MODE[3], command = lambda: set_mode(b'MD03;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # CW-U
tk.Button(f2, text = MODE[4], command = lambda: set_mode(b'MD04;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # FM
tk.Button(f3, text = MODE[5], command = lambda: set_mode(b'MD06;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # RTTY

tk.Button(f1, text = MODE[6], command = lambda: set_mode(b'MD0E;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # C4FM
tk.Button(f2, text = MODE[7], command = lambda: set_mode(b'MD0C;'), width=10, fg='white', bg='green').pack(padx=1, pady=1) # DATA-USB
button_a = tk.Button(f3, textvariable=var1, command = lambda: reset(), width=10, bg='yellow')
button_a.pack(padx=1, pady=1) 

# BANDボタン設定 3X4
tk.Button(f1, text = BAND[0], command = lambda: set_band(b'BS00;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f2, text = BAND[1], command = lambda: set_band(b'BS01;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f3, text = BAND[2], command = lambda: set_band(b'BS03;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)

tk.Button(f1, text = BAND[3], command = lambda: set_band(b'BS04;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f2, text = BAND[4], command = lambda: set_band(b'BS05;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f3, text = BAND[5], command = lambda: set_band(b'BS06;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)

tk.Button(f1, text = BAND[6], command = lambda: set_band(b'BS07;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f2, text = BAND[7], command = lambda: set_band(b'BS08;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f3, text = BAND[8], command = lambda: set_band(b'BS09;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)

tk.Button(f1, text = BAND[9], command = lambda: set_band(b'BS10;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f2, text = BAND[10], command = lambda: set_band(b'BS15;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)
tk.Button(f3, text = BAND[11], command = lambda: set_band(b'BS16;'), width=10, fg='white', bg='blue').pack(padx=1, pady=1)

# フィルタなど設定
tk.Button(f4, text = 'Normal', command = lambda: set_narrow('d'), width=10, bg='cyan').pack(padx=1, pady=1) # 
tk.Button(f4, text = 'Narrow',  command = lambda: set_narrow('n'), width=10, bg='cyan').pack(padx=1, pady=1) # 

ez_sp = tk.Button(f4, text = 'Tiny Split', width=10) # Tiny Split Mode
ez_sp.pack(padx=1, pady=1)

rit_button = tk.Button(f4, text = ' RIT ', command = lambda: set_rit(), width=10)
rit_button.pack(padx=1, pady=1) # RIT
xit_button = tk.Button(f4, text = ' XIT ', command = lambda: set_xit(), width=10)
xit_button.pack(padx=1, pady=1) # XIT

tk.Button(f4, text = 'Clear', command = lambda: clear(), width=10).pack(padx=1, pady=1)
button_b = tk.Button(f4, textvariable=var4, command = lambda: set_sq(), width=10)
button_b.pack(padx=1, pady=1)

th1 = threading.Thread(target=read_cmd)
th1.start()

th2 = threading.Thread(target=send_if_cmd, daemon=True)
th2.start()

button_a.bind('<MouseWheel>', wheel1)
button_b.bind('<MouseWheel>', wheel2)

ez_sp.bind('<Button-1>', ez_split_on)
ez_sp.bind('<ButtonRelease-1>', ez_split_off)

init()

var4.set('SQL')

root.mainloop()
exit_flag = True
ser.close()
